/*!
    \file keynames.h
    \brief Key/button naming functions
    
		Converts strings to SDLK/SDL_BUTTON values and vice versa
*/
#ifndef _KEYNAMES_H
#define _KEYNAMES_H


#ifdef __WIN32__
#include <windows.h>
#endif

#ifndef __APPLE__
#include <SDL/SDL.h>
#else
#include <SDL.h>
#endif

/**
	\brief converts a button name string to a integer representing the corresponding SDL mouse button identifier
	\param name: the name of the mouse button
	\return an int containing the SDL identifier of the mouse button or -1 if the button name is not valid
*/
int buttonnameToSDLB( char* name);

/**
	\brief converst a SDL mouse button identifier to a name string
	\param button: an SDL mouse button identifier
	\return a pointer to a string containing the name of the mouse button
*/
char* SDLBToButtonname( int button);

/**
	\brief converts a key name string to a integer representing the corresponding SDLK sym
	\param name: the name of the key
	\return the SDLK sym of the named key or -1 if the key name is not valid
*/
int keynameToSDLK( char* name);

/**
	\brief converts an SDLK sym to a name string
	\param key: the SDLK sym
	\return a pointer to a string containig the name of the key
*/
char* SDLKToKeyname( int key);


#endif /* _KEYNAMES_H */
